@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: iso2022.c,v 1.22 2003/02/27 00:31:47 nalin Exp $"
+#ident "$Id: iso2022.c,v 1.23 2003/04/14 19:41:12 nalin Exp $"
 #include "../config.h"
 #include <sys/types.h>
 #include <stdio.h>
@@ -279,7 +279,7 @@ _vte_iso2022_is_ambiguous(gunichar c)
 		}
 	}
 	p = GINT_TO_POINTER(c);
-	return g_tree_lookup(ambiguous, p) != NULL;
+	return g_tree_lookup(ambiguous, p) == p;
 }
 
 struct _vte_iso2022 *
@@ -551,11 +551,12 @@ _vte_iso2022_substitute_single(gunichar mapname, gunichar c)
 				(long) c);
 		}
 #endif
-	} else
-	if (g_unichar_iswide(result)) {
-		width = 2;
 	} else {
-		width = 1;
+		if (g_unichar_iswide(result)) {
+			width = 2;
+		} else {
+			width = 1;
+		}
 	}
 	/* Encode the width. */
 	result = _vte_iso2022_set_encoded_width(result, width);
@@ -996,6 +997,19 @@ _vte_iso2022_substitute(struct _vte_iso2022 *outside_state,
 	return j;
 }
 
+gssize
+_vte_iso2022_unichar_width(gunichar c)
+{
+	c = c & ~(VTE_ISO2022_ENCODED_WIDTH_MASK); /* just in case */
+	if (_vte_iso2022_is_ambiguous(c)) {
+		return _vte_iso2022_ambiguous_width();
+	}
+	if (g_unichar_iswide(c)) {
+		return 2;
+	}
+	return 1;
+}
+
 #ifdef ISO2022_MAIN
 static void
 debug_print(FILE *fp, const char *string)
