@@ -19,7 +19,7 @@
 #ifndef vte_vte_h_included
 #define vte_vte_h_included
 
-#ident "$Id: vte.h,v 1.59 2003/01/09 06:56:01 nalin Exp $"
+#ident "$Id: vte.h,v 1.64 2003/03/07 00:28:21 nalin Exp $"
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -114,11 +114,12 @@ typedef enum {
 } VteTerminalEraseBinding;
 
 /* The structure we return as the supplemental attributes for strings. */
-struct vte_char_attributes {
+struct _VteCharAttributes {
 	long row, column;
 	GdkColor fore, back;
 	gboolean underline:1, strikethrough:1;
 };
+typedef struct _VteCharAttributes VteCharAttributes;
 
 /* The widget's type. */
 GtkType vte_terminal_get_type(void);
@@ -172,6 +173,7 @@ void vte_terminal_set_audible_bell(VteTerminal *terminal, gboolean is_audible);
 gboolean vte_terminal_get_audible_bell(VteTerminal *terminal);
 void vte_terminal_set_visible_bell(VteTerminal *terminal, gboolean is_visible);
 gboolean vte_terminal_get_visible_bell(VteTerminal *terminal);
+void vte_terminal_set_scroll_background(VteTerminal *terminal, gboolean scroll);
 void vte_terminal_set_scroll_on_output(VteTerminal *terminal, gboolean scroll);
 void vte_terminal_set_scroll_on_keystroke(VteTerminal *terminal,
 					  gboolean scroll);
@@ -196,6 +198,8 @@ void vte_terminal_set_default_colors(VteTerminal *terminal);
 void vte_terminal_set_background_image(VteTerminal *terminal, GdkPixbuf *image);
 void vte_terminal_set_background_image_file(VteTerminal *terminal,
 					    const char *path);
+void vte_terminal_set_background_tint_color(VteTerminal *terminal,
+					    const GdkColor *color);
 void vte_terminal_set_background_saturation(VteTerminal *terminal,
 					    double saturation);
 void vte_terminal_set_background_transparent(VteTerminal *terminal,
@@ -246,7 +250,7 @@ void vte_terminal_reset(VteTerminal *terminal, gboolean full,
 /* Read the contents of the terminal, using a callback function to determine
  * if a particular location on the screen (0-based) is interesting enough to
  * include.  Each byte in the returned string will have a corresponding
- * struct vte_char_attributes in the passed GArray, if the array was not NULL.
+ * VteCharAttributes structure in the passed GArray, if the array was not NULL.
  * Note that it will have one entry per byte, not per character, so indexes
  * should match up exactly. */
 char *vte_terminal_get_text(VteTerminal *terminal,
@@ -274,6 +278,9 @@ void vte_terminal_match_clear_all(VteTerminal *terminal);
 /* Add a matching expression, returning the tag the widget assigns to that
  * expression. */
 int vte_terminal_match_add(VteTerminal *terminal, const char *match);
+/* Set the cursor to be used when the pointer is over a given match. */
+void vte_terminal_match_set_cursor(VteTerminal *terminal, int tag,
+				   GdkCursor *cursor);
 /* Remove a matching expression by tag. */
 void vte_terminal_match_remove(VteTerminal *terminal, int tag);
 
