@@ -18,7 +18,7 @@
 
 /* The interfaces in this file are subject to change at any time. */
 
-#ident "$Id: vtedraw.c,v 1.3 2003/03/07 17:51:21 nalin Exp $"
+#ident "$Id: vtedraw.c,v 1.6 2003/04/14 23:31:59 nalin Exp $"
 
 #include "../config.h"
 #include <stdio.h>
@@ -27,17 +27,35 @@
 #include <gtk/gtk.h>
 #include "debug.h"
 #include "vtedraw.h"
+#include "vteft2.h"
+#include "vtegl.h"
 #include "vtepango.h"
+#include "vtepangox.h"
 #include "vteskel.h"
 #include "vtexft.h"
 
 struct _vte_draw_impl
 *_vte_draw_impls[] = {
 	&_vte_draw_skel,
+#if GTK_CHECK_VERSION(2,2,0)
+#ifndef X_DISPLAY_MISSING
+#ifdef HAVE_GL
+	/* &_vte_draw_gl, */
+#endif
+#endif
+#endif
+#ifndef X_DISPLAY_MISSING
 #ifdef HAVE_XFT2
 	&_vte_draw_xft,
 #endif
+#endif
+	&_vte_draw_ft2,
 	&_vte_draw_pango,
+#ifndef X_DISPLAY_MISSING
+#ifdef HAVE_PANGOX
+	&_vte_draw_pango_x,
+#endif
+#endif
 };
 
 struct _vte_draw *
@@ -98,6 +116,22 @@ _vte_draw_free(struct _vte_draw *draw)
 	g_free(draw);
 }
 
+GdkVisual *
+_vte_draw_get_visual(struct _vte_draw *draw)
+{
+	g_return_val_if_fail(draw->impl != NULL, NULL);
+	g_return_val_if_fail(draw->impl->get_visual != NULL, NULL);
+	return draw->impl->get_visual(draw);
+}
+
+GdkColormap *
+_vte_draw_get_colormap(struct _vte_draw *draw)
+{
+	g_return_val_if_fail(draw->impl != NULL, NULL);
+	g_return_val_if_fail(draw->impl->get_colormap != NULL, NULL);
+	return draw->impl->get_colormap(draw);
+}
+
 void
 _vte_draw_start(struct _vte_draw *draw)
 {
@@ -169,11 +203,19 @@ _vte_draw_get_text_height(struct _vte_draw *draw)
 }
 
 int
-_vte_draw_get_text_base(struct _vte_draw *draw)
+_vte_draw_get_text_ascent(struct _vte_draw *draw)
 {
 	g_return_val_if_fail(draw->impl != NULL, 1);
-	g_return_val_if_fail(draw->impl->get_text_base != NULL, 1);
-	return draw->impl->get_text_base(draw);
+	g_return_val_if_fail(draw->impl->get_text_ascent != NULL, 1);
+	return draw->impl->get_text_ascent(draw);
+}
+
+gboolean
+_vte_draw_get_using_fontconfig(struct _vte_draw *draw)
+{
+	g_return_val_if_fail(draw->impl != NULL, 1);
+	g_return_val_if_fail(draw->impl->get_using_fontconfig != NULL, FALSE);
+	return draw->impl->get_using_fontconfig(draw);
 }
 
 void
@@ -184,8 +226,7 @@ _vte_draw_text(struct _vte_draw *draw,
 	g_return_if_fail(draw->started == TRUE);
 	g_return_if_fail(draw->impl != NULL);
 	g_return_if_fail(draw->impl->draw_text != NULL);
-	draw->impl->draw_text(draw, requests, n_requests,
-			      color, alpha);
+	draw->impl->draw_text(draw, requests, n_requests, color, alpha);
 }
 
 void
@@ -210,15 +251,6 @@ _vte_draw_draw_rectangle(struct _vte_draw *draw,
 	draw->impl->draw_rectangle(draw, x, y, width, height, color, alpha);
 }
 
-gboolean
-_vte_draw_scroll(struct _vte_draw *draw, gint dx, gint dy)
-{
-	g_return_val_if_fail(draw->started == TRUE, FALSE);
-	g_return_val_if_fail(draw->impl != NULL, FALSE);
-	g_return_val_if_fail(draw->impl->scroll != NULL, FALSE);
-	return draw->impl->scroll(draw, dx, dy);
-}
-
 void
 _vte_draw_set_scroll(struct _vte_draw *draw, gint x, gint y)
 {
